/*
 * QuickTime for Java SDK Sample Code

   Usage subject to restrictions in SDK License Agreement
 * Copyright:  1996-1999 Apple Computer, Inc.

 */
import java.awt.*;
import java.awt.event.*;

import quicktime.*;
import quicktime.sound.*;
import quicktime.util.*;

public class SoundMemRecord extends Frame implements WindowListener, Errors, SoundConstants {	
	private static int
		kOSType =  1 << 1;
	private static int sndBufferSize = 327680;
	
	public static void main (String args[]) {
		SoundMemRecord sr = new SoundMemRecord ("Sound Recording");
		sr.show();
		sr.toFront();
	}

	SndHandle hdl;
	SPBDevice snd;

	SoundMemRecord (String title) {
		super (title);
		try {
			QTSession.open();
			snd = new SPBDevice (null, siWritePermission);
			System.out.println ("OptionsDialog==" + snd.hasOptionsDialog());
			System.out.println ("NumChannels=" + snd.getChannelAvailable());
			System.out.println ("SampleSize=" + snd.getSampleSize());
			printArray ("Names", snd.getInputSourceNames());
			printArray("Compression", snd.getCompressionAvailable(), kOSType);
			printArray("SampleSize", snd.getSampleSizeAvailable(), 0);
			printArray("SampleRate", snd.getSampleRateAvailable());
			snd.setPlayThruOnOff (0);
			hdl = new SndHandle (snd.getNumberChannels(), 
											snd.getSampleRate(), 
											snd.getSampleSize(), 
											snd.getCompressionType(), 
											60);
			hdl.appendSoundBuffer (sndBufferSize);	//this is the size of our sound buffer
		} catch (Exception ee) {
			ee.printStackTrace();
			QTSession.close();
		}
		

		setLayout(new GridLayout(1, 3, 2, 2));
		add (startButton);
		add (playButton);

		startButton.addActionListener (new ActionListener () {
			public void actionPerformed (ActionEvent event) {
				try{					
					System.out.println ("StartRecording");
					SPB recorder = new SPB(snd, 0, snd.bytesToMilliseconds(sndBufferSize), hdl.getSoundData());
					recorder.record (false);
					System.out.println ("FinishedRecording");
					hdl.setupHeader (snd.getNumberChannels(), 
											snd.getSampleRate(), 
											snd.getSampleSize(), 
											snd.getCompressionType(), 
											60,
											recorder.getCount());
				} catch (QTException ee){
					ee.printStackTrace();
				}	
		}
		});

		playButton.addActionListener (new ActionListener () {
			public void actionPerformed (ActionEvent event) {
				try{
					if (hdl != null) Sound.play (hdl);
				} catch (QTException ee){
					ee.printStackTrace();
				}	
			}
		});
		addNotify();
		Insets insets = getInsets();
		setBounds (0, 0, (insets.left + insets.right + 220), (insets.top + insets.bottom + 16));
		addWindowListener (this);
	}
	
	private static void printArray (String prefix, int[] ar, int printTypeFlag) {
		System.out.print (prefix + "=[");
		if (ar.length == 0) {
			System.out.println ("]");
			return;
		}
		for (int i = 0; i < ar.length - 1; i++) {
			if ((printTypeFlag & kOSType) != 0)
				System.out.print (QTUtils.fromOSType(ar[i]) + ",");
			else
				System.out.print (ar[i] + ",");
		}
		if ((printTypeFlag & kOSType) != 0)
			System.out.println (QTUtils.fromOSType(ar[ar.length-1]) + "]");
		else
			System.out.println (ar[ar.length-1] + "]");
	}
	
	private static void printArray (String prefix, float[] ar) {
		System.out.print (prefix + "=[");
		if (ar.length == 0) {
			System.out.println ("]");
			return;
		}
		for (int i = 0; i < ar.length - 1; i++) {
			System.out.print (ar[i] + "F,");
		}
		System.out.println (ar[ar.length-1] + "F]");
	}

	private static void printArray (String prefix, String[] ar) {
		System.out.print (prefix + "=[");
		if (ar.length == 0) {
			System.out.println ("]");
			return;
		}
		for (int i = 0; i < ar.length - 1; i++) {
			System.out.print (ar[i] + ",");
		}
		System.out.println (ar[ar.length-1] + "]");
	}

	private Button startButton = new Button("Record");
	private Button playButton = new Button("Play");
	private boolean recording = false;
		
	public void windowClosing (WindowEvent e) {
		QTSession.close();
		dispose();
	}

	public void windowClosed (WindowEvent e) { 
		System.exit(0);
	}
	
	public void windowIconified (WindowEvent e) {}
	public void windowDeiconified (WindowEvent e) {}
	public void windowOpened (WindowEvent e) {}
	public void windowActivated (WindowEvent e) {}
	public void windowDeactivated (WindowEvent e) {}
}
